@@ -4,7 +4,7 @@
 use std::{
     env,
     fs::File,
-    mem,
+    io, mem,
     sync::{
         atomic::{AtomicBool, Ordering},
         Arc,
@@ -127,12 +127,26 @@ fn main() {
 
         let file = File::open(&arg).expect("fopen");
 
+        let uncompressed: Box<dyn io::Read + Send> = if arg.ends_with(".zst") {
+            Box::new(zstd::Decoder::new(file).expect("zst decoder"))
+        } else if arg.ends_with(".bz2") {
+            Box::new(bzip2::read::MultiBzDecoder::new(file))
+        } else if arg.ends_with(".xz") {
+            Box::new(xz2::read::XzDecoder::new(file))
+        } else if arg.ends_with(".gz") {
+            Box::new(flate2::read::GzDecoder::new(file))
+        } else if arg.ends_with(".lz4") {
+            Box::new(lz4::Decoder::new(file).expect("lz4 decoder"))
+        } else {
+            Box::new(file)
+        };
+
         let mut validator = Validator::new();
         let (send, recv) = crossbeam::channel::bounded(128);
 
         crossbeam::scope(|scope| {
             scope.spawn(move |_| {
-                for game in BufferedReader::new(file).into_iter(&mut validator) {
+                for game in BufferedReader::new(uncompressed).into_iter(&mut validator) {
                     send.send(game.expect("io")).unwrap();
                 }
             });
